import {AnimationLoop, ClipSpace} from 'luma.gl';

const INFO_HTML = `
<p>
  <b>Animating Quasicrystals</b>
<p>
  Crystal patterns generated by wavefront interference patterns.
  Rendered by a custom fragment shader in a luma.gl <code>ClipSpace</code> model.
  A luma.gl port (of the PhiloGL port) of the work of
  <a href="http://www.jasondavies.com/animated-quasicrystals/">Jason Davies</a>
  <form action="#">
    <label for="wavefronts">Wavefronts </label>
    <input id="wavefronts" type="range" value="7.0" min="1" max="10" step="0.1"/>
  </form>
`;

const FRAGMENT_SHADER = `\
precision highp float;

#define PI 3.1415926535

uniform float uTime;
uniform float uRatio;

varying vec2 position;

void main(void) {
  vec2 defpixel = (position - vec2( 0.5 ) ) * 170.;

  float step = PI / uRatio;

  // Sum up total of all waves
  float total;
  for (float i = 0.; i < 100.; i++) {
    if ( i < uRatio ) {
      float value = i * step;
      float s = sin( value );
      float c = cos( value );
      total += ( cos( c * defpixel.x + s * defpixel.y + uTime ) + 1. ) / 2.;
    }
  }

  float v = mod(total, 1.);
  float k = total - v;
  total = ( mod( abs( k ), 2. ) ) <= 0.0001 ? v : 1. - v;

  gl_FragColor =
    vec4( total * (1. - (uRatio / 20.)), total * (uRatio / 10.), total * (uRatio / 5.), 1. );
}
`;

const animationLoop = new AnimationLoop({
  onInitialize: ({gl}) => {
    return {clipSpace: new ClipSpace(gl, {fs: FRAGMENT_SHADER})};
  },

  onRender: ({gl, canvas, time, clipSpace}) => {
    clipSpace.draw({
      uniforms: {
        uTime: (time / 600) % (Math.PI * 2),
        uRatio: animationLoop.getHTMLControlValue('wavefronts', 7)
      }
    });
  },

  onAddHTML() {
    return INFO_HTML;
  }
});

animationLoop.getInfo = () => INFO_HTML;

export default animationLoop;

/* global window */
if (!window.website) {
  animationLoop.start();
}
